@@ -1,8 +1,10 @@
 from typing import Callable
 from typing import Dict
 from typing import List
+from typing import Union
 
 from .http_method import HttpMethod
+from .middleware import MiddlewarePipeline
 from .routing import Route
 from .routing import Router
 from .serverless import IS_SERVERLESS_ENVIRONMENT
@@ -17,15 +19,18 @@ class HttpApplication:
     schema - for handling json schema
     timeout - for time - outing
     """
-    def __init__(self) -> None:
-        self.middleware: List[Middleware] = []
+    def __init__(self, *middleware: Union[Middleware, Callable]):
+
+        self.middleware = MiddlewarePipeline()
+        for item in middleware:
+            self.middleware.append(item)
         self.methods: Dict[HttpMethod, Router] = {key: Router() for key in HttpMethod}
 
     def get(self, route: str) -> Callable:
         def _get(handler: Callable) -> Callable:
             self.methods[HttpMethod.GET].append(Route(route), handler)
             if IS_SERVERLESS_ENVIRONMENT:
-                return make_serverless_callback(handler, Route(route))
+                return make_serverless_callback(self.middleware, handler, Route(route))
             return handler
 
         return _get
@@ -34,7 +39,7 @@ class HttpApplication:
         def _post(handler: Callable) -> Callable:
             self.methods[HttpMethod.POST].append(Route(route), handler)
             if IS_SERVERLESS_ENVIRONMENT:
-                return make_serverless_callback(handler, Route(route))
+                return make_serverless_callback(self.middleware, handler, Route(route))
             return handler
 
         return _post
@@ -43,7 +48,7 @@ class HttpApplication:
         def _put(handler: Callable) -> Callable:
             self.methods[HttpMethod.PUT].append(Route(route), handler)
             if IS_SERVERLESS_ENVIRONMENT:
-                return make_serverless_callback(handler, Route(route))
+                return make_serverless_callback(self.middleware, handler, Route(route))
             return handler
 
         return _put
@@ -52,7 +57,7 @@ class HttpApplication:
         def _patch(handler: Callable) -> Callable:
             self.methods[HttpMethod.PATCH].append(Route(route), handler)
             if IS_SERVERLESS_ENVIRONMENT:
-                return make_serverless_callback(handler, Route(route))
+                return make_serverless_callback(self.middleware, handler, Route(route))
             return handler
 
         return _patch
@@ -61,7 +66,7 @@ class HttpApplication:
         def _delete(handler: Callable) -> Callable:
             self.methods[HttpMethod.DELETE].append(Route(route), handler)
             if IS_SERVERLESS_ENVIRONMENT:
-                return make_serverless_callback(handler, Route(route))
+                return make_serverless_callback(self.middleware, handler, Route(route))
             return handler
 
         return _delete
@@ -70,7 +75,7 @@ class HttpApplication:
         def _head(handler: Callable) -> Callable:
             self.methods[HttpMethod.HEAD].append(Route(route), handler)
             if IS_SERVERLESS_ENVIRONMENT:
-                return make_serverless_callback(handler, Route(route))
+                return make_serverless_callback(self.middleware, handler, Route(route))
             return handler
 
         return _head
@@ -79,7 +84,7 @@ class HttpApplication:
         def _options(handler: Callable) -> Callable:
             self.methods[HttpMethod.OPTIONS].append(Route(route), handler)
             if IS_SERVERLESS_ENVIRONMENT:
-                return make_serverless_callback(handler, Route(route))
+                return make_serverless_callback(self.middleware, handler, Route(route))
             return handler
 
         return _options
@@ -94,13 +99,10 @@ class HttpApplication:
             self.methods[HttpMethod.HEAD].append(Route(route), handler)
             self.methods[HttpMethod.OPTIONS].append(Route(route), handler)
             if IS_SERVERLESS_ENVIRONMENT:
-                return make_serverless_callback(handler, Route(route))
+                return make_serverless_callback(self.middleware, handler, Route(route))
             return handler
 
         return _any
 
 
-http = HttpApplication()
-
-
-__all__ = ["HttpApplication", "http"]
+__all__ = ["HttpApplication"]
